@@ -1,14 +1,14 @@
 
 #' Anticlustering
 #'
-#' Partition a pool of elements into groups (i.e., anticlusters) with the aim 
-#' of creating high within-group heterogeneity and high between-group similarity.
-#' Anticlustering is accomplished by maximizing instead
-#' of minimizing a clustering objective function. 
-#' Implements anticlustering algorithms as described in
-#' Papenberg and Klau (2021; <doi:10.1037/met0000301>) and an implementation
-#' of the bicriterion anticlustering algorithm described by Brusco, Cradit 
-#' and Steinley (2020; <doi:10.1111/bmsp.12186>).
+#' Partition a pool of elements into groups (i.e., anticlusters) with
+#' the aim of creating high within-group heterogeneity and high
+#' between-group similarity.  Anticlustering is accomplished by
+#' maximizing instead of minimizing a clustering objective function.
+#' Implements anticlustering methods as described in Papenberg and
+#' Klau (2021; <doi:10.1037/met0000301>), Brusco et al. 
+#' (2020; <doi:10.1111/bmsp.12186>), and Papenberg (2023; 
+#' <doi:10.31234/osf.io/7jw6v>).
 #'
 #' @param x The data input. Can be one of two structures: (1) A
 #'     feature matrix where rows correspond to elements and columns
@@ -18,36 +18,38 @@
 #'     \code{\link{dist}} or \code{\link{as.dist}}) or a \code{matrix}
 #'     where the entries of the upper and lower triangular matrix
 #'     represent pairwise dissimilarities.
-#' @param K How many anticlusters should be created. Alternatively: (a) 
-#'     A vector describing the size of each group, or (b) a
-#'     vector of length \code{nrow(x)} describing how elements are
-#'     assigned to anticlusters before the optimization starts.
+#' @param K How many anticlusters should be created. Alternatively:
+#'     (a) A vector describing the size of each group, or (b) a vector
+#'     of length \code{nrow(x)} describing how elements are assigned
+#'     to anticlusters before the optimization starts.
 #' @param objective The objective to be maximized. The options
 #'     "diversity" (default; previously called "distance", which is
-#'     still supported), "variance", "kplus" and "dispersion" are natively
-#'     supported. May also be a user-defined function. See Details.
-#' @param method One of "exchange" (default) , "local-maximum", "brusco", or
-#'     "ilp".  See Details.
+#'     still supported), "variance", "kplus" and "dispersion" are
+#'     natively supported. May also be a user-defined function. See
+#'     Details.
+#' @param method One of "exchange" (default) , "local-maximum",
+#'     "brusco", or "ilp".  See Details.
 #' @param preclustering Boolean. Should a preclustering be conducted
 #'     before anticlusters are created? Defaults to \code{FALSE}. See
 #'     Details.
 #' @param categories A vector, data.frame or matrix representing one
-#'     or several categorical constraints. See Details.
+#'     or several categorical variables whose distribution should be similar 
+#'     between groups. See Details.
 #' @param repetitions The number of times a search heuristic is
 #'     initiated when using \code{method = "exchange"}, \code{method =
-#'     "local-maximum"}, or \code{method = "brusco"}. In the end, 
-#'     the best objective found across the repetitions is returned.
-#' @param standardize Boolean. If \code{TRUE} and \code{x} is a feature 
-#'     matrix, the data is standardized through a call to \code{\link{scale}}
-#'     before the optimization starts. This argument is silently ignored 
-#'     if \code{x} is a distance matrix.
+#'     "local-maximum"}, or \code{method = "brusco"}. In the end, the
+#'     best objective found across the repetitions is returned.
+#' @param standardize Boolean. If \code{TRUE} and \code{x} is a
+#'     feature matrix, the data is standardized through a call to
+#'     \code{\link{scale}} before the optimization starts. This
+#'     argument is silently ignored if \code{x} is a distance matrix.
 #'     
 #'
 #' @return A vector of length N that assigns a group (i.e, a number
 #'     between 1 and \code{K}) to each input element.
 #'
 #' @importFrom Matrix sparseMatrix
-#' @importFrom stats as.dist dist
+#' @importFrom stats as.dist dist sd
 #'
 #' @export
 #'
@@ -56,31 +58,33 @@
 #'
 #' @details
 #'
-#' This function is used to solve anticlustering. That is, the data input is 
-#' divided into \code{K} in such a way that elements within groups are heterogeneous 
-#' and the different groups are similar. Anticlustering is accomplished by maximizing
-#' instead of minimizing a clustering objective function. The
-#' maximization of four clustering objective functions is natively
-#' supported (other functions can also defined by the user as
-#' described below):
+#' This function is used to solve anticlustering. That is, the data
+#' input is divided into \code{K} groups in such a way that elements
+#' within groups are heterogeneous and the different groups are
+#' similar. Anticlustering is accomplished by maximizing instead of
+#' minimizing a clustering objective function. The maximization of
+#' four clustering objective functions is natively supported (other
+#' functions can also defined by the user as described below):
 #' 
 #' \itemize{
-#'   \item{cluster editing `diversity` objective, setting \code{objective = "diversity"} (this is the default objective)}
+#'   \item{cluster editing `diversity` objective, setting 
+#'         \code{objective = "diversity"} (this is the default objective)}
 #'   \item{k-means `variance` objective, setting \code{objective = "variance"}}
 #'   \item{`k-plus` objective, an extension of the k-means variance criterion,
 #'         setting \code{objective = "kplus"}}
-#'   \item{the `dispersion` objective, which the minimum distance between 
-#'         any two elements within the same cluster.}
+#'   \item{the `dispersion` objective is the minimum distance between 
+#'         any two elements within the same cluster (setting 
+#'         \code{objective = "dispersion"})}
 #' }
 #'
 #' The k-means objective is the within-group variance---that is, the
 #' sum of the squared distances between each element and its cluster
 #' center (see \code{\link{variance_objective}}). K-means
 #' anticlustering focuses on minimizing differences with regard to the
-#' means of the input variables \code{x}; k-plus \code{objective = "kplus"}
-#' anticlustering is an extension of this criterion that also tries to
-#' minimize differences with regard to the standard deviations between
-#' groups (see \code{\link{kplus_objective}}).
+#' means of the input variables \code{x}; k-plus \code{objective =
+#' "kplus"} anticlustering is an extension of this criterion that also
+#' tries to minimize differences with regard to the standard
+#' deviations between groups (see \code{\link{kplus_anticlustering}}).
 #' 
 #' The cluster editing "diversity" objective is the sum of pairwise
 #' distances of elements within the same groups (see
@@ -88,19 +92,27 @@
 #' known as the »maximum diverse grouping problem« because it
 #' maximizes group diversity as measured by the sum of pairwise
 #' distances. Hence, anticlustering maximizes between-group similarity
-#' by maximizing within-group heterogeneity. In previous versions of
-#' this package, \code{method = "distance"} was used (and is still
-#' supported) to request anticluster editing, but now \code{method =
-#' "diversity"} is preferred because there are several clustering
-#' objectives based on pairwise distances (e.g., see
-#' \code{\link{dispersion_objective}}).
+#' by maximizing within-group heterogeneity. Note that the equivalence
+#' of within-group heterogeneity and between-group similarity only
+#' holds for equal-sized groups in the case of anticluster editing. 
+#' If you request different-sized groups
+#' and wish the different groups to be similar to each other, use
+#' \code{objective = "kplus"} or \code{objective = "variance"}. In
+#' previous versions of this package, \code{method = "distance"} was
+#' used (and is still supported) to request anticluster editing, but
+#' now \code{method = "diversity"} is preferred because there are
+#' several clustering objectives based on pairwise distances (e.g.,
+#' see \code{\link{dispersion_objective}}).
 #' 
 #' The "dispersion" is the minimum distance between any two elements
 #' that are part of the same cluster; maximization of this objective
 #' ensures that any two elements within the same group are as
 #' dissimilar from each other as possible. Applications that require
 #' high within-group heterogeneity often require to maximize the
-#' dispersion. 
+#' dispersion. Oftentimes, it is useful to also consider the diversity
+#' and not only the dispersion; to optimize both objectives at the
+#' same time, see the function
+#' \code{\link{bicriterion_anticlustering}}.
 #'
 #' If the data input \code{x} is a feature matrix (that is: each row
 #' is a "case" and each column is a "variable") and the option
@@ -111,13 +123,13 @@
 #'
 #' In the standard case, groups of equal size are generated. Adjust
 #' the argument \code{K} to create groups of different size (see
-#' examples).
+#' Examples).
 #'
 #' \strong{Algorithms for anticlustering}
 #'
 #' By default, a heuristic method is employed for anticlustering: the
-#' exchange method (\code{method = "exchange"}). First, anticlusters
-#' are randomly assigned to clusters. (It is also possible to
+#' exchange method (\code{method = "exchange"}). First, elements
+#' are randomly assigned to anticlusters (It is also possible to
 #' explicitly specify the initial assignment using the argument
 #' \code{K}; in this case, \code{K} has length \code{nrow(x)}.) Based
 #' on the initial assignment, elements are systematically swapped
@@ -140,11 +152,15 @@
 #' on the preclustering heuristic follows below). This option is
 #' recommended for larger N.
 #' 
-#' Using \code{method = "brusco"} implements the local bicriterion 
-#' iterated local search heuristic by Brusco et al. (2020), but only optimizes
-#' for one criterion - diversity or dispersion.
-#' \code{\link{bicriterion_anticlustering}}) can be used to conduct the actual 
-#' bicriterion optimization that is offered by Brusco et al.'s algorithm.
+#' Using \code{method = "brusco"} implements the local bicriterion
+#' iterated local search (BILS) heuristic by Brusco et al. (2020) and returns
+#' the partition that best optimized either the diversity or the dispersion
+#' during the optimization process. The function
+#' \code{\link{bicriterion_anticlustering}} can also be used to run the algorithm
+#' by Brusco et al., but it returns multiple partitions that approximate the 
+#' optimal pareto efficient set according to both objectives (diversity and 
+#' dispersion). Thus, to fully utilize the BILS algorithm, use the function 
+#' \code{\link{bicriterion_anticlustering}}.
 #'
 #' \strong{Optimal anticlustering}
 #'
@@ -169,12 +185,6 @@
 #' approach applicable for larger problem instances. With
 #' preclustering, optimality is no longer guaranteed, but the solution
 #' is usually optimal or very close to optimal.
-#'
-#' The variance and dispersion criterion cannot be optimized to optimality using
-#' integer linear programming because the objective functions are not linear. 
-#' However, it is possible to employ the function
-#' \code{\link{generate_partitions}} to obtain optimal solutions for
-#' small problem instances.
 #' 
 #' \strong{Preclustering}
 #' 
@@ -192,13 +202,15 @@
 #' minimum pairwise distance by solving the integer linear program
 #' described in Papenberg and Klau (2021; (8) - (10), (12) - (13)).
 #' 
-#' \strong{Categorical constraints}
+#' \strong{Categorical variables}
 #'
-#' The argument \code{categories} may induce categorical constraints.
-#' The grouping variables indicated by \code{categories} will be
-#' balanced out across anticlusters. This functionality is
-#' only available for the classical exchange procedures, that is, for 
-#' \code{method = "exchange"} and \code{method = "local-maximum"}.
+#' The argument \code{categories} may induce categorical constraints,
+#' i.e., can be used to distribute categorical variables evenly
+#' between sets.  The grouping variables indicated by
+#' \code{categories} will be balanced out across anticlusters. This
+#' functionality is only available for the classical exchange
+#' procedures, that is, for \code{method = "exchange"} and
+#' \code{method = "local-maximum"}.
 #' 
 #' \strong{Optimize a custom objective function}
 #' 
@@ -216,7 +228,7 @@
 #' that in the function body, columns of the data set cannot be
 #' accessed using \code{data.frame} operations such as
 #' \code{$}. Objects of class \code{dist} will be converted to matrix
-#' as well. 
+#' as well.
 #' 
 #' 
 #' @examples
@@ -299,6 +311,9 @@
 #' Papenberg, M., & Klau, G. W. (2021). Using anticlustering to partition 
 #' data sets into equivalent parts. Psychological Methods, 26(2), 
 #' 161–174. https://doi.org/10.1037/met0000301.
+#' 
+#' Papenberg, M. (2023). k-plus Anticlustering: An Improved k-means Criterion 
+#' for Maximizing Between-Group Similarity. Retrieved from psyarxiv.com/7jw6v 
 #'
 #' Späth, H. (1986). Anticlustering: Maximizing the variance criterion.
 #' Control and Cybernetics, 15, 213-218.
@@ -350,15 +365,7 @@ anticlustering <- function(x, K, objective = "diversity", method = "exchange",
     return(repeat_anticlustering(x, K, objective, categories, method, repetitions))
   }
   
-  ## Get data into required format
-  input_validation_anticlustering(x, K, objective, method, preclustering, 
-                                  categories, repetitions)
 
-  ## Exact method using ILP
-  if (method == "ilp") {
-    return(exact_anticlustering(x, K, preclustering))
-  }
-  
   if (method == "brusco") {
     if (objective == "diversity") {
       weights <- c(0.5, 0.99, 0.999, 0.999999)
@@ -372,11 +379,7 @@ anticlustering <- function(x, K, objective = "diversity", method = "exchange",
     best_obj <- which.max(apply(partitions, 1, obj_fun, convert_to_distances(x)))
     return(partitions[best_obj, ])
   }
-  
-  # Preclustering and categorical constraints are both processed in the
-  # variable `categories` after this step:
-  categories <- get_categorical_constraints(x, K, preclustering, categories)
-  
+
   if (inherits(objective, "function")) {
     # most generic exchange method, deals with any objective function
     return(exchange_method(x, K, objective, categories))
